--
-- AttachedVehicleSupportAnim
-- Specialization to change the supportAnimation of attachedVehicle if the vehicle is attached/detached 
-- Used to stop walking of vehicle
--
-- @author John Deere 6930
-- @date 03/12/2020
--

--[[XML

    <attacherJoints ... changeSupportAnimOfAttachedVehicle="BOOL" invertSupportAnimOfAttachedVehicle="BOOL">
        ...
	</attacherJoints>

]]

AttachedVehicleSupportAnim = {};
AttachedVehicleSupportAnim.modDir = g_currentModDirectory;
AttachedVehicleSupportAnim.MOD_NAME = g_currentModName;

function AttachedVehicleSupportAnim.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AttacherJoints, specializations);
end;

function AttachedVehicleSupportAnim.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "updateAttachedSupportAnimations", AttachedVehicleSupportAnim.updateAttachedSupportAnimations);
end;

function AttachedVehicleSupportAnim.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", AttachedVehicleSupportAnim);
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", AttachedVehicleSupportAnim);
    SpecializationUtil.registerEventListener(vehicleType, "onPreAttach", AttachedVehicleSupportAnim);
    SpecializationUtil.registerEventListener(vehicleType, "onPostAttach", AttachedVehicleSupportAnim);
    SpecializationUtil.registerEventListener(vehicleType, "onPostDetach", AttachedVehicleSupportAnim);
end;

function AttachedVehicleSupportAnim:onLoad(savegame)
    self.spec_attachedVehicleSupportAnim = self[string.format("spec_%s.attachedVehicleSupportAnim", AttachedVehicleSupportAnim.MOD_NAME)];
    local spec = self.spec_attachedVehicleSupportAnim;	
    
 	spec.changeSupportAnimOfAttachedVehicle = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.attacherJoints#changeSupportAnimOfAttachedVehicle"), false);
 	spec.invertSupportAnimOfAttachedVehicle = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.attacherJoints#invertSupportAnimOfAttachedVehicle"), false);
    
    self.initAttachedVehicles = false;
    self.initAttachedVehiclesTime = g_currentMission.time+2000;
end;

function AttachedVehicleSupportAnim:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    if not self.initAttachedVehicles and self.initAttachedVehiclesTime < g_currentMission.time then
        if self:getAttacherVehicle() == nil then
            self:updateAttachedSupportAnimations(1);
        end;
        self.initAttachedVehicles = true;
    end;
end

function AttachedVehicleSupportAnim:onPreAttach(attacherVehicle, inputJointDescIndex, jointDescIndex)
    self:updateAttachedSupportAnimations(-1);
end;

function AttachedVehicleSupportAnim:onPostAttach(attacherVehicle, inputJointDescIndex, jointDescIndex)
    self:updateAttachedSupportAnimations(-1);
end;

function AttachedVehicleSupportAnim:onPostDetach()
    self:updateAttachedSupportAnimations(1);
end;

function AttachedVehicleSupportAnim:updateAttachedSupportAnimations(direction)
    local spec = self.spec_attachedVehicleSupportAnim;
    if spec.changeSupportAnimOfAttachedVehicle then
        local invert = 1;
        if spec.invertSupportAnimOfAttachedVehicle then
            invert = -1;
        end;
        for _, implement in pairs(self.spec_attacherJoints.attachedImplements) do
            if implement.object ~= nil then
                if implement.object.spec_attachable ~= nil then
                    for _, supportAnimation in ipairs(implement.object.spec_attachable.supportAnimations) do
                        if implement.object:getIsSupportAnimationAllowed(supportAnimation) then
                            implement.object:playAnimation(supportAnimation.animationName, direction*invert, nil, true);
                            
                            if implement.object.propertyState == Vehicle.PROPERTY_STATE_SHOP_CONFIG or loadFromSavegame then
                                AnimatedVehicle.updateAnimationByName(implement.object, supportAnimation.animationName, 9999999);
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;
